@@ -1,7 +1,7 @@
 import asyncio
 import click
 from functools import wraps
-from .web import create_app
+from .web import run_with_gunicorn
 from .config import Config
 
 
@@ -18,7 +18,7 @@ def async_cmd(func):
 def main(ctx, config):
     ctx.ensure_object(dict)
 
-    ctx.obj['config'] = config
+    Config.load(config)
 
 
 @main.command()
@@ -27,20 +27,7 @@ def serve(ctx):
     """
     Start the http relay server 
     """
-    app = create_app(ctx.obj['config'])
-    
-    from gunicorn.app.base import Application
-
-    class ASGIApplication(Application):
-        def load_config(self):
-            self.cfg.set('worker_class', 'uvicorn.workers.UvicornWorker')
-            for k, v in Config.gunicorn.items():
-                self.cfg.set(k.lower(), v)
-
-        def load(self):
-            return app
-
-    ASGIApplication().run()
+    run_with_gunicorn()
 
 
 @main.command()
@@ -59,7 +46,6 @@ async def adduser(ctx, identifier='', pubkey='', relay=None):
         click.echo(f'Adding {identifier} = {pubkey} with relays: {relay}')
 
         from .db import get_storage
-        Config.load(ctx.obj['config'])
         async with get_storage() as storage:
             await storage.set_identified_pubkey(identifier, pubkey, relays=relay)
 
@@ -70,17 +56,22 @@ async def adduser(ctx, identifier='', pubkey='', relay=None):
 @click.pass_context
 @async_cmd
 async def query(ctx, query, results):
+    """
+    Run a REQ query and display results
+    """
     import rapidjson
     import asyncio
     from .db import get_storage, Subscription
-    Config.load(ctx.obj['config'])
     if not query:
         click.echo("query is required")
         return -1
+    query = query.strip()
+    if not query.startswith('['):
+        query = f'[{query}]'
     query = rapidjson.loads(query)
     queue = asyncio.Queue()
     async with get_storage() as storage:
-        sub = Subscription(storage.db, 'cli', query, queue=queue)
+        sub = Subscription(storage.db, 'cli', query, queue=queue, default_limit=60000)
         sub.prepare()
         click.echo(click.style('Query:', bold=True))
         click.echo(click.style(sub.query, fg="green"))
@@ -89,10 +80,52 @@ async def query(ctx, query, results):
     if results:
         click.echo(click.style('Results:', fg='black', bold=True))
 
-        while not queue.empty():
+        while True:
             sub, event = await queue.get()
             if event:
                 click.echo(click.style(rapidjson.dumps(rapidjson.loads(event), indent=4), fg="red"))
                 click.echo('')
+            else:
+                break
 
 
+@main.command()
+@click.option("--query", '-q', help="Query", prompt="Enter REQ filters", default='[{"kinds":[0,1,2,3,4,5,5,6,7,8,9]}]')
+@click.pass_context
+@async_cmd
+async def update_tags(ctx, query):
+    """
+    Update the tags in the tag table, from a REQ query
+    """
+    import rapidjson
+    import asyncio
+    from .event import Event
+    from .db import get_storage, Subscription
+
+    if not query:
+        query = '[{}]'
+
+    query = rapidjson.loads(query)
+    queue = asyncio.Queue()
+
+    async with get_storage() as storage:
+        sub = Subscription(storage.db, 'cli', query, queue=queue, default_limit=600000)
+        sub.prepare()
+        click.echo(click.style('Query:', bold=True))
+        click.echo(click.style(sub.query, fg="green"))
+        await sub.run_query()
+
+        count = 0
+        async with storage.db.cursor() as cursor:
+
+            while True:
+                sub, event_json = await queue.get()
+                if event_json:
+                    event = Event(**rapidjson.loads(event_json))
+                    await storage.process_tags(cursor, event)
+                    count += 1
+                else:
+                    break
+
+        await storage.db.commit()
+        click.echo("Processed %d events" % count)
