@@ -59,18 +59,23 @@ class Storage:
 
     async def close(self):
         await self.verifier.stop()
+        await self.db.executescript("""
+            PRAGMA analysis_limit=400;
+            PRAGMA optimize;
+        """)
         await self.db.close()
-
+        
     async def setup_db(self):
         LOG.info(f"Database file {self.filename} {'exists' if os.path.exists(self.filename) else 'does not exist'}")
         async with aiosqlite.connect(self.filename) as db:
             await migrate(db)
         self.db = await aiosqlite.connect(self.filename)
         await self.db.executescript('''
-            pragma journal_mode=wal;
-            pragma synchronous = normal;
-            pragma temp_store = memory;
-            pragma mmap_size = 30000000000;
+            PRAGMA journal_mode=wal;
+            PRAGMA synchronous = normal;
+            PRAGMA temp_store = memory;
+            PRAGMA mmap_size = 30000000000;
+            PRAGMA foreign_keys = ON;
         ''')
         await self.verifier.start(self.db)
         self.garbage_collector = start_garbage_collector()
@@ -130,6 +135,8 @@ class Storage:
             raise StorageException("invalid: Bad signature")
         if (time.time() - event.created_at) > Config.oldest_event:
             raise StorageException(f"invalid: {event.created_at} is too old")
+        elif (time.time() - event.created_at) < -3600:
+            raise StorageException(f"invalid: {event.created_at} is in the future")
 
     async def pre_save(self, cursor, event):
         """
@@ -146,7 +153,7 @@ class Storage:
 
         if event.is_replaceable:
             # check for older event from same pubkey
-            await cursor.execute('select id, created_at from event where pubkey = ? and kind = ? and created_at < ?', (event.pubkey, event.kind, event.created_at))
+            await cursor.execute('select id, created_at from event where pubkey = ? and kind = ? and created_at <= ?', (event.pubkey, event.kind, event.created_at))
             row = await cursor.fetchone()
             if row:
                 old_id = row[0]
@@ -155,6 +162,24 @@ class Storage:
                 await cursor.execute('delete from event where id = ?', (old_id, ))
         return True
 
+    async def process_tags(self, cursor, event):
+        if event.tags:
+            # update mentions
+            # single-letter tags can be searched
+            # delegation tags are also searched
+            # expiration tags are also added for the garbage collector
+            tags = set((event.id_bytes, tag[0], tag[1]) for tag in event.tags if tag[0] in ('delegation', 'expiration') or len(tag[0]) == 1)
+            if tags:
+                await cursor.executemany('INSERT OR IGNORE INTO tag (id, name, value) VALUES (?, ?, ?)', tags)
+
+            if event.kind == EventKind.DELETE:
+                # delete the referenced events
+                for tag in event.tags:
+                    name = tag[0]
+                    if name == 'e':
+                        event_id = tag[1]
+                        await cursor.execute('DELETE FROM event WHERE id = ? AND pubkey = ?', (bytes.fromhex(event_id), event.pubkey))
+
     async def post_save(self, cursor, event):
         """
         Post-process event
@@ -167,21 +192,7 @@ class Storage:
                 query = 'DELETE FROM event WHERE pubkey = ? AND kind = ? AND created_at < ?'
                 LOG.debug("q:%s kind:%s, key:%s", query, event.kind, event.pubkey)
                 await cursor.execute(query, (event.pubkey, event.kind, event.created_at))
-            elif event.kind in (EventKind.TEXT_NOTE, EventKind.ENCRYPTED_DIRECT_MESSAGE) and event.tags:
-                # update mentions
-                # single-letter tags can be searched
-                # delegation tags are also searched
-                # expiration tags are also added for the garbage collector
-                tags = set((event.id_bytes, tag[0], tag[1]) for tag in event.tags if tag[0] in ('delegation', 'expiration') or len(tag[0]) == 1)
-                if tags:
-                    await cursor.executemany('INSERT OR IGNORE INTO tag (id, name, value) VALUES (?, ?, ?)', tags)
-            elif event.kind == EventKind.DELETE and event.tags:
-                # delete the referenced events
-                for tag in event.tags:
-                    name = tag[0]
-                    if name == 'e':
-                        event_id = tag[1]
-                        await cursor.execute('DELETE FROM event WHERE id = ? AND pubkey = ?', (bytes.fromhex(event_id), event.pubkey))
+            await self.process_tags(cursor, event)
         else:
             LOG.debug("skipped post-processing for %s", event)
 
@@ -280,13 +291,14 @@ class Storage:
 
 
 class Subscription:
-    def __init__(self, db, sub_id, filters:list, queue=None, client_id=None):
+    def __init__(self, db, sub_id, filters:list, queue=None, client_id=None, default_limit=5000):
         self.db  = db
         self.sub_id = sub_id
         self.client_id = client_id
         self.filters = filters
         self.queue = queue
         self.query_task = None
+        self.default_limit = default_limit
 
     def prepare(self):
         try:
@@ -409,7 +421,7 @@ class Subscription:
                 else:
                     raise ValueError("until")
             elif key == 'limit' and value:
-                filter_obj['limit'] = max(min(int(value or 0), 5000), 0)
+                filter_obj['limit'] = max(min(int(value or 0), self.default_limit), 0)
             elif key[0] == '#' and len(key) == 2 and value:
                 pstr = []
                 for val in set(value):
@@ -456,7 +468,7 @@ class Subscription:
             select += '\n) OR (\n'.join(where)
             select += ')'
         if limit is None:
-            limit = 5000
+            limit = self.default_limit
         select += f'''
             ORDER BY created_at DESC LIMIT {limit}
         '''
